"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var _autoprefixer = require("autoprefixer");
var CleanWebpackPlugin = require("clean-webpack-plugin");
var CopyPlugin = require("copy-webpack-plugin");
var ExtractTextPlugin = require("extract-text-webpack-plugin");
var HtmlWebpackPlugin = require("html-webpack-plugin");
var path_1 = require("path");
var _webpack = require("webpack");
var _loaders = require("./loaders");
var loaders_1 = require("./loaders");
var LoaderOptionsPlugin = _webpack.LoaderOptionsPlugin, DefinePlugin = _webpack.DefinePlugin, UglifyJsPlugin = _webpack.optimize.UglifyJsPlugin;
tslib_1.__exportStar(require("./synchronize"), exports);
tslib_1.__exportStar(require("./node"), exports);
/**
 * Loaders ref.
 */
exports.loaders = tslib_1.__assign({}, _loaders);
/**
 * Plugins ref.
 */
exports.plugins = {
    ExtractTextPlugin: ExtractTextPlugin,
    HtmlWebpackPlugin: HtmlWebpackPlugin,
    CopyPlugin: CopyPlugin,
    CleanWebpackPlugin: CleanWebpackPlugin,
};
/**
 * Autoprefix ref.
 */
exports.autoprefixer = _autoprefixer;
/**
 * Webpack ref.
 */
exports.webpack = _webpack;
/**
 * Create a default webpack configuration.
 *
 * ```ts
 * import { createConfiguration } from "oly-tools";
 *
 * export default createConfiguration({
 *   entry: "./src/main.browser.tsx",
 * });
 * ```
 *
 * Even if it's cool to have a ready2go webpack configuration,
 * you should avoid this function and make your own configuration.
 *
 * ```ts
 * export default (env: string) => {
 *
 *  const config = createConfiguration({
 *   entry: "./src/main.browser.tsx",
 *  });
 *
 *  // edit config here
 *
 *  return config;
 * }
 * ```
 *
 * @param options {IToolsOptions}
 * @return {Configuration}
 */
function createConfiguration(options) {
    var config = {};
    var root = options.root || process.cwd();
    var env = typeof options.env === "object" ? options.env : {};
    var isProduction = (process.argv.indexOf("-p") > -1)
        || process.env.NODE_ENV === "production"
        || options.production === true
        || (!!env.production || env.NODE_ENV === "production");
    env.NODE_ENV = isProduction ? "production" : "development";
    options.extract = options.extract !== false;
    options.entry = options.entry || "./src/index.ts";
    options.dist = options.dist || path_1.resolve(root, "www");
    options.template = options.template || path_1.resolve(__dirname, "index.html");
    options.hash = typeof options.hash === "undefined" ? isProduction : options.hash;
    options.fontLoader = options.fontLoader || loaders_1.fontLoaderFactory(options);
    options.imageLoader = options.imageLoader || loaders_1.imageLoaderFactory(options);
    options.typescriptLoader = options.typescriptLoader || loaders_1.typescriptLoaderFactory();
    options.styleLoader = options.styleLoader || loaders_1.cssLoaderFactory();
    // define the format of the source-map
    // set to false for disabled it
    config.devtool =
        typeof options.sourceMaps === "undefined"
            ? (isProduction ? false : "source-map")
            : options.sourceMaps;
    // set a default context (base)
    // it's the root directory for all relative path
    config.context = root;
    // main entry (or entries)
    // can be a single file or an object of file
    config.entry = options.entry;
    // resolve provides options to navigate into node_modules/sources
    config.resolve = {
        extensions: [".webpack.js", ".js", ".web.js", ".ts", ".tsx", ".json"],
        mainFields: ["webpack", "browser", "module", "main"],
        modules: [
            "node_modules",
            path_1.join(root, "node_modules"),
            path_1.join(__dirname, "../node_modules"),
        ],
    };
    config.resolveLoader = {
        modules: config.resolve.modules,
    };
    config.output = {
        filename: options.hash ? "[name].[hash].js" : "[name].js",
        path: options.dist,
        publicPath: "/",
    };
    if (options.exclude) {
        var exclude_1 = options.exclude;
        config.externals = [
            function (context, request, callback) {
                if (exclude_1.test(request)) {
                    var chunks = request.split("/");
                    return callback(null, "{" + chunks[chunks.length - 1] + ": function Empty() {}};");
                }
                callback();
            },
        ];
    }
    // Loaders
    config.module = {
        rules: [
            options.typescriptLoader,
            options.styleLoader,
            options.fontLoader,
            options.imageLoader,
            {
                test: /\.json$/,
                use: "json-loader",
            },
        ],
    };
    // Plugins
    config.plugins = [
        // important for react, useful for projects
        // 'process.env.NODE_ENV' will be replaced by true/false
        new DefinePlugin({
            "process.env": Object
                .keys(env)
                .reduce(function (o, key) {
                o[key] = JSON.stringify(env[key]);
                return o;
            }, {}),
        }),
        // extract css from js
        new ExtractTextPlugin({
            disable: !options.extract,
            filename: options.hash ? "[name].[hash].css" : "[name].css",
        }),
        // create index.html for us
        new HtmlWebpackPlugin({
            template: options.template,
        }),
    ];
    if (options.assets) {
        config.plugins.push(new CopyPlugin([{
                from: options.assets, to: "./",
            }]));
    }
    if (isProduction) {
        config.bail = true;
        config.plugins.push(new LoaderOptionsPlugin({
            debug: false,
            minimize: true,
        }), 
        // remove outdir
        new CleanWebpackPlugin([options.dist], {
            root: root,
            verbose: false,
        }));
        config.plugins.push(new UglifyJsPlugin({
            comments: false,
            mangle: {
                keep_fnames: true,
            },
        }));
    }
    // Axios & some universal libs use Buffer in their code
    // Webpack see this and try to emulate Buffer with a big code
    config.node = {
        Buffer: false,
    };
    // Dev Server
    config.devServer = {
        historyApiFallback: true,
        inline: true,
        stats: "errors-only",
    };
    return config;
}
exports.createConfiguration = createConfiguration;
//# sourceMappingURL=index.js.map